// pari_xl.cpp : Defines the entry point for the DLL application.

#include "stdafx.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved)
{
    return TRUE;
}


#include "stdafx.h"
#include <malloc.h>
#include <memory.h>
#include <string.h>

#include "pari.h"
#include "paripriv.h"
#include "parigen.h"

long PARIinitialized;

#define maximalLength 100000 // length of strings for PARI in or out, approx 20 pages

#include <stdlib.h>
#include <string.h>


/*
	eval_pari:
	Take a command as string for the first argument, hand it over to PARI to evaluate
	it. Write the result (converted to a string) in the second argument, if no errors
	occured and return the length of the resulting string.
	In case of numerical results there is some string manipulation, s.th. Maple finally
	should be able to parse that to Maple numerics. The global resultNumeric is set to 1.
	In case of an error write PARI's error message to the second argument and return
	the _negative_ length of that string. The global resultNumeric is set to 0.

	Assumes that Pari is initialized and resultStr has (enough) allocated memory
*/
long eval_pari(char* resultStr, char* inputStr)
{
	GEN y;
	char* outputBuf;
	long inputLen = 0;
	long outputLen = 0;
	char* p_error = "[PARI error message]:\n";

	inputLen = strlen(inputStr);
	if ( inputLen <= 0 )
		{
		memmove(resultStr, "\0", 1);	// resultStr[0] =  "\0";
		return 0;
		}

	outputBuf = (char *)calloc( maximalLength, sizeof( char ) );;

	CATCH(CATCH_ALL)
  { /* we have caught an exception ! */
    CATCH_RELEASE();
		free(outputBuf);
		outputLen = strlen(errmessage[pari_errno]);
		memmove(resultStr, p_error, strlen(p_error));
		strcat(resultStr, errmessage[pari_errno]);
		return  (outputLen + strlen(p_error));
  }
	TRY
  {
	y = gp_read_str(inputStr);	// compute with PARI
	brute(y,'e',-1);						// raw format: libpari.pdf, 4.7.2 output, p.29 & users.pdf, 2.11.16 output, p.58
	outputBuf = GENtostr( y );	// revert to C string

	outputLen = strlen( outputBuf );
  }
  ENDCATCH;

	if ( outputLen <= 0 )
		{
		memmove(resultStr, inputStr, inputLen);
		free(outputBuf);
		return 0;
		}

	memmove(resultStr, outputBuf, outputLen);
	free(outputBuf);
	return outputLen;
}

long __stdcall start_pari()
{
	if (PARIinitialized == 1){;}	// already alive? else start with defaults
	else
	{
		pari_init(1000000,2);
		PARIinitialized = 1;
	}
	return PARIinitialized;
}

long __stdcall stop_pari()
{
	if (PARIinitialized == 0){;}	// already already killed? else shut down
	else
	{
		pari_close();
		PARIinitialized = 0;
	}
	return PARIinitialized;
}


long __stdcall call_pari( char* outStr, char* inStr, unsigned int inLength)
{
	char* inBuf;
	long inLen;
	char* outBuf;
	long outLen;
	long i, iLast, iStart=0;

	if (PARIinitialized == 1){;}	// already alive? else start with defaults
	else
	{
		pari_init(1000000,2);
		PARIinitialized = 1;
	}

	outBuf = (char *)calloc( maximalLength, sizeof( char ) );

	inLen = min(maximalLength,strlen(inStr));

	// convert the input VBA string to a C string, VBA char is 2 bytes
	inBuf = (char *)calloc(inLength,sizeof(char) + 1);
	iLast = inLength + 1;

	for(i=iStart;i<iLast;i++){
		inBuf[i] = inStr[2*i];}
	inBuf[inLength] = '\0';

	outLen = eval_pari(outBuf,inBuf);	// calling Pari to evaluate
	free(inBuf);

	// now build a VBA string of minimal length holding the result
	// since call is by reference the result is seen in the calling VBA
	for(i=0;i<outLen;i++){
		outStr[2*i] = outBuf[i];
		if (outBuf[i] == '\0')
			{break;}
		}
	free(outBuf);

	return i;
}